import asyncio
import logging
import re
from typing import List, Optional
from bs4 import BeautifulSoup
from ..client.aiohttp import AiohttpClient
from ..types.models import File, Order
from ..const import Get, Post, USER_AGENT

logger = logging.getLogger(__name__)


class API:
    def __init__(self, login: str = None, password: str = None, session: str = None):
        self.__login: str = login
        self.__password: str = password
        self.__session: str = session
        self.http_client: AiohttpClient = AiohttpClient()

    async def login(self) -> None:
        """Авторизация через login и password и сохранение sessionid."""
        if not self.__login or not self.__password:
            raise ValueError("Login and password are required for authentication.")

        response = await self.http_client.request_raw(
            url=Post.LOGIN,
            method="POST",
            headers={
                "User-Agent": USER_AGENT,
                "Referer": Post.LOGIN,
            },
            data={"login": self.__login, "password": self.__password},
        )

        session_cookie = response.cookies.get("session")
        if session_cookie:
            self.__session = session_cookie.value
        else:
            raise ValueError("Session ID not found. Login failed.")

    async def fetch_order_details(self, order_index: int) -> Optional[str]:
        """Получает описание заказа (description)."""
        url = Get.FETCH_ORDERS
        data = {"action": "view", "index": str(order_index)}
        headers = {
            "User-Agent": USER_AGENT,
            "Referer": url,
            "Cookie": f"session={self.__session}",
        }

        response = await self.http_client.request_raw(
            url, method="GET", data=data, headers=headers
        )

        if response.status != 200:
            logger.error(
                f"Failed to fetch order details for {order_index}: {response.status}"
            )
            return None

        html = await response.text()
        soup = BeautifulSoup(html, "html.parser")

        desc_tag = soup.find("tr", text="Description")
        return (
            desc_tag.find_next_sibling("td").get_text(strip=True) if desc_tag else None
        )

    async def get_order_files(self, order_index: int) -> List[File]:
        """Получает список файлов, прикрепленных к заказу."""
        url = f"https://4writers.net/index1.php?mode=files&order={order_index}"
        headers = {
            "User-Agent": USER_AGENT,
            "Referer": url,
            "Cookie": f"session={self.__session}",
        }

        response = await self.http_client.request_raw(url, method="GET", headers=headers)

        if response.status != 200:
            logger.error(f"Failed to fetch files for order {order_index}: {response.status}")
            return []

        html = await response.text()
        soup = BeautifulSoup(html, "html.parser")

        # Находим файлы
        file_rows = soup.select("tbody tr")
        files = []
        for row in file_rows:
            cols = row.find_all("td")
            if len(cols) < 3:
                continue

            file_date = cols[0].text.strip()
            file_author = cols[1].text.strip()
            file_link_tag = cols[2].find("a")

            if file_link_tag and "href" in file_link_tag.attrs:
                file_name = file_link_tag.text.strip()
                file_id_match = re.search(r"item=(\d+)", file_link_tag["href"])
                file_id = int(file_id_match.group(1)) if file_id_match else None

                if file_id:
                    files.append(File(date=file_date, author=file_author, name=file_name, id=file_id))

        return files

    async def download_file(self, file_id: int) -> bytes:
        """
        Скачивает файл по `file_id` и возвращает его содержимое в `bytes`.
        :param file_id: Уникальный ID файла.
        :return: Байты файла (`bytes`). Если ошибка — возвращает `bytes()`.
        """
        url = f"https://4writers.net/download.php?mode=files&item={file_id}"
        headers = {
            "User-Agent": USER_AGENT,
            "Referer": url,
            "Cookie": f"session={self.__session}",
        }

        response = await self.http_client.request_raw(url, method="GET", headers=headers)

        if response.status != 200:
            logger.error(f"Failed to download file {file_id}: {response.status}")
            return bytes()  # Возвращаем пустые байты при ошибке

        file_bytes = await response.read()

        logger.info(f"File {file_id} downloaded successfully ({len(file_bytes)} bytes)")
        return file_bytes  # Возвращаем байты файла

    async def take_order(self, order_index: int) -> bool:
        """
        Берёт заказ по `order_index`.
        :param order_index: ID заказа, который нужно взять.
        :return: `True`, если заказ успешно взят, иначе `False`.
        """
        url = "https://4writers.net/index2.php?mode=free_orders"
        data = {"action": "take", "index": str(order_index)}
        headers = {
            "User-Agent": USER_AGENT,
            "Referer": url,
            "Cookie": f"session={self.__session}",
        }

        response = await self.http_client.request_raw(url, method="POST", data=data, headers=headers)

        if response.status != 200:
            logger.error(f"Failed to take order {order_index}: {response.status}")
            return False

        response_text = await response.text()

        if "Now you can write this order" in response_text:
            logger.info(f"Successfully took order {order_index}")
            return True
        else:
            logger.warning(f"Failed to take order {order_index}. Response: {response_text}")
            return False

    async def get_orders(self) -> Optional[List[Order]]:
        """Получает список заказов, их описание и прикрепленные файлы (но не скачивает их)."""
        response = await self.http_client.request_raw(
            url=Get.ORDERS,
            method="GET",
            headers={
                "User-Agent": USER_AGENT,
                "Referer": Get.ORDERS,
                "Cookie": f"session={self.__session}",
            },
        )

        if response.status != 200:
            logger.error(f"Failed to fetch orders: {response.status}")
            return None

        html_content = await response.text()
        soup = BeautifulSoup(html_content, "html.parser")

        order_items = soup.find_all("div", class_="order-item")
        if not order_items:
            logger.info("No orders found.")
            return []

        field_mapping = {
            "title": lambda order: order.find("div", class_="order_title").text.strip(),
            "subject": lambda order: order.find("div", class_="order_subject").text.strip(),
            "order_id": lambda order: order.find("div", class_="order_id").find("span", class_="value").text.strip(),
            "order_index": lambda order: int(re.search(r"index:\s*'(\d+)'", order.find("a", text="View")["onclick"]).group(1)),
            "deadline": lambda order: order.find("div", class_="deadline").find("span", class_="value").text.strip(),
            "remaining": lambda order: order.find("div", class_="remaining").text.strip(),
            "order_type": lambda order: order.find("div", class_="label_cell", text="Order type").find_next_sibling("div").text.strip(),
            "academic_level": lambda order: order.find("div", class_="label_cell", text="Academic level").find_next_sibling("div").text.strip(),
            "style": lambda order: order.find("div", class_="label_cell", text="Style").find_next_sibling("div").text.strip(),
            "language": lambda order: order.find("div", class_="label_cell", text="Language").find_next_sibling("div").text.strip(),
            "pages": lambda order: int(order.find("div", class_="label_cell", text="Pages").find_next_sibling("div").text.strip()),
            "sources": lambda order: int(order.find("div", class_="label_cell", text="Sources").find_next_sibling("div").text.strip()),
            "salary": lambda order: float(order.find("div", class_="label_cell", text="Salary").find_next_sibling("div").text.strip().replace("$", "")),
            "bonus": lambda order: float(order.find("div", class_="label_cell", text="Bonus").find_next_sibling("div").text.strip().replace("$", "")),
            "total": lambda order: float(order.find("div", class_="label_cell", text="Total").find_next_sibling("div").text.strip().replace("$", "")),
        }

        orders = []
        detail_tasks = []
        file_tasks = []

        for order_html in order_items:
            try:
                order_obj = Order(
                    **{key: extractor(order_html) for key, extractor in field_mapping.items()},
                    description=None,  # Будет заполнено позже
                    files=None,  # Будет заполнено позже
                )
                orders.append(order_obj)

                detail_tasks.append(self.fetch_order_details(order_obj.order_index))  # Запрос описания
                file_tasks.append(self.get_order_files(order_obj.order_index))  # Запрос файлов

            except Exception as e:
                logger.warning(f"Error creating Order object: {e}")

        # Запускаем все запросы параллельно
        descriptions = await asyncio.gather(*detail_tasks)
        files_list = await asyncio.gather(*file_tasks)

        # Заполняем заказы описанием и списком файлов
        for order, desc, files in zip(orders, descriptions, files_list):
            order.description = desc
            order.files = files if files else None  # `None`, если файлов нет

        return orders

